@@ -1,4 +1,3 @@
-from keplergl import KeplerGl
 from pathlib import Path
 from pkg_resources import resource_filename
 import json
@@ -7,6 +6,14 @@ import numpy as np
 import os
 import sys
 
+# kepler is optional, check if installed.
+try:
+    from keplergl import KeplerGl
+except:
+    has_kepler = False
+else:
+    has_kepler = True
+
 
 class HiddenPrints:
     """Disables stdout prints for a block of code."""
@@ -38,23 +45,35 @@ def is_valid_lon(val: str) -> bool:
         return False
 
 
+def is_not_none(val: any) -> bool:
+    return val is not None
+
+
 def read_csv_file(filename):
     """Read input csv file, dropping rows that don't have valid location data."""
-    df = pd.read_csv(filename)
-    initial_rows = len(df)
+    # replace NaN for all fields not to break kepler parsing.
+    df = pd.read_csv(filename).replace({np.nan: None})
 
-    df = df.dropna(subset=["lat", "lon"])
-    df = df.replace(
-        {np.nan: None}
-    )  # replace for other fields not to break kepler parsing
-    print(f"Ignored {initial_rows - len(df)} coordinates with NaN")
+    # construct an index of values with valid lat & lon.
+    valid_index = df.lat.apply(is_valid_lat) & df.lon.apply(is_valid_lon)
+    df_invalid = df[~valid_index]
+
+    count_invalid = len(df_invalid)
+    if count_invalid:
+        df_not_empty = df_invalid[
+            (df_invalid.lat.apply(is_not_none) | df_invalid.lon.apply(is_not_none))
+        ]
+
+        count_not_empty = len(df_not_empty)
+        count_empty = count_invalid - count_not_empty
+
+        if count_empty:
+            print(f"Removed {count_empty} empty coordinate pairs.")
+
+        if count_not_empty:
+            print(f"Removed {count_not_empty} invalid coordinate pairs:")
+            print(df_not_empty[["lat", "lon"]].to_string())
 
-    valid_index = df.lat.astype(str).apply(is_valid_lat) & df.lon.astype(str).apply(
-        is_valid_lon
-    )
-    if len(df_invalid := df[~valid_index]):
-        print(f"Found {len(df_invalid)} invalid coordinate pairs, ignoring:")
-        print(df_invalid[["lat", "lon"]].to_string())
     return df[valid_index]
 
 
@@ -77,6 +96,10 @@ def write_output_file(dirname, filename, data):
 
 def write_visualization(dirname, filename, data):
     """Write a visualization, ensuring parent directories."""
+
+    if not has_kepler:
+        return None
+
     # Hide kepler stdout output.
     with HiddenPrints():
         map = KeplerGl()
